@@ -169,16 +169,9 @@ impl StyledToken {
         return StyledToken { token, style };
     }
 
+    // Are all characters in this token whitespace?
     pub fn is_whitespace(&self) -> bool {
-        let mut chars_iterator = self.token.chars();
-        let first_char = chars_iterator.next().unwrap();
-        if chars_iterator.next().is_some() {
-            // Multiple chars found in this token, but whitespace will only be
-            // one per token.
-            return false;
-        }
-
-        return first_char.is_whitespace();
+        return self.token.chars().all(|c| c.is_whitespace());
     }
 }
 
@@ -259,13 +252,38 @@ pub fn render(line_style: &LineStyle, prefix: &str, tokens: &[StyledToken]) -> S
 ///
 /// Returns true if something was unhighlighted, false otherwise.
 pub fn unhighlight_noisy_rows(tokens: &mut [StyledToken]) -> bool {
-    fn maybe_unhighlight_row(row: &mut [StyledToken], highlighted_tokens_count: usize) -> bool {
-        if row.is_empty() {
+    fn maybe_unhighlight_row(row: &mut [StyledToken]) -> bool {
+        // Count highlighted and not highlighted tokens, ignoring leading whitespace
+        let mut in_indentation = true;
+        let mut highlighted_chars_count = 0;
+        let mut all_chars_count = 0;
+        let mut highlighted_tokens_count = 0;
+        let mut all_tokens_count = 0;
+        for token in row.iter() {
+            if in_indentation {
+                if token.is_whitespace() {
+                    // Ignore indentation when counting the highlighting percentage
+                    continue;
+                }
+                in_indentation = false;
+            }
+
+            if token.style == Style::Highlighted {
+                highlighted_chars_count += token.token.chars().count();
+                highlighted_tokens_count += 1;
+            }
+            all_chars_count += token.token.chars().count();
+            all_tokens_count += 1;
+        }
+
+        if all_chars_count == 0 {
             return false;
         }
 
-        let highlighted_percentage = (100 * highlighted_tokens_count) / row.len();
-        if highlighted_percentage <= 70 {
+        let highlighted_chars_percentage = (100 * highlighted_chars_count) / all_chars_count;
+        let highlighted_tokens_percentage = (100 * highlighted_tokens_count) / all_tokens_count;
+        if highlighted_chars_percentage < 70 || highlighted_tokens_percentage < 50 {
+            // Little enough of the line is highlighted, this is fine
             return false;
         }
 
@@ -276,7 +294,6 @@ pub fn unhighlight_noisy_rows(tokens: &mut [StyledToken]) -> bool {
         return true;
     }
 
-    let mut highlighted_tokens_count = 0;
     let mut line_start_index = 0;
     let mut changed = false;
 
@@ -284,27 +301,21 @@ pub fn unhighlight_noisy_rows(tokens: &mut [StyledToken]) -> bool {
         let token = &tokens[i];
         if token.token == "\n" {
             // End of line, evaluate!
-            changed |=
-                maybe_unhighlight_row(&mut tokens[line_start_index..i], highlighted_tokens_count);
+            changed |= maybe_unhighlight_row(&mut tokens[line_start_index..i]);
 
             // Reset for the next row
             line_start_index = i + 1;
-            highlighted_tokens_count = 0;
             continue;
         }
-
-        if token.style == Style::Highlighted {
-            highlighted_tokens_count += 1;
-        }
     }
 
     // Handle the last row
-    changed |= maybe_unhighlight_row(&mut tokens[line_start_index..], highlighted_tokens_count);
+    changed |= maybe_unhighlight_row(&mut tokens[line_start_index..]);
 
     return changed;
 }
 
-pub fn highlight_trailing_whitespace(tokens: &mut [StyledToken]) {
+pub fn errorlight_trailing_whitespace(tokens: &mut [StyledToken]) {
     let mut in_trailer = true;
     for token in tokens.iter_mut().rev() {
         if token.token == "\n" {
@@ -321,7 +332,7 @@ pub fn highlight_trailing_whitespace(tokens: &mut [StyledToken]) {
     }
 }
 
-pub fn highlight_nonleading_tabs(tokens: &mut [StyledToken]) {
+pub fn errorlight_nonleading_tabs(tokens: &mut [StyledToken]) {
     let mut leading = true;
     for token in tokens.iter_mut() {
         if token.token == "\n" {
@@ -339,7 +350,9 @@ pub fn highlight_nonleading_tabs(tokens: &mut [StyledToken]) {
     }
 }
 
-pub fn unhighlight_leading_whitespace(tokens: &mut [StyledToken]) {
+/// Returns true if something was unhighlighted, false otherwise.
+pub fn unhighlight_leading_whitespace(tokens: &mut [StyledToken]) -> bool {
+    let mut i_did_it = false;
     let mut in_leading = true;
     for token in tokens.iter_mut() {
         if token.token == "\n" {
@@ -350,12 +363,15 @@ pub fn unhighlight_leading_whitespace(tokens: &mut [StyledToken]) {
         if in_leading && token.is_whitespace() {
             if token.style == Style::Highlighted {
                 token.style = Style::Plain;
+                i_did_it = true;
             }
             continue;
         }
 
         in_leading = false;
     }
+
+    return i_did_it;
 }
 
 pub(crate) fn align_tabs(old: &mut [StyledToken], new: &mut [StyledToken]) {
@@ -502,7 +518,6 @@ pub fn lowlight_timestamp(row: &mut [StyledToken]) {
     #[derive(PartialEq)]
     enum State {
         Initial,
-        FoundOneSpace,
         InTimestamp,
     }
 
@@ -510,17 +525,9 @@ pub fn lowlight_timestamp(row: &mut [StyledToken]) {
     for token in row.iter_mut() {
         match state {
             State::Initial => {
-                if token.token == "\t" {
+                let is_multispace = token.token.len() > 1 && token.token.chars().all(|c| c == ' ');
+                if token.token == "\t" || is_multispace {
                     state = State::InTimestamp;
-                } else if token.token == " " {
-                    state = State::FoundOneSpace;
-                }
-            }
-            State::FoundOneSpace => {
-                if token.token == " " {
-                    state = State::InTimestamp;
-                } else {
-                    state = State::Initial;
                 }
             }
             State::InTimestamp => {
@@ -587,7 +594,7 @@ mod tests {
     use pretty_assertions::assert_eq;
 
     #[test]
-    fn test_highlight_nonleading_tabs() {
+    fn test_errorlight_nonleading_tabs() {
         let mut tokens = vec![
             StyledToken::new("\t".to_string(), Style::Plain),
             StyledToken::new("\t".to_string(), Style::Plain),
@@ -596,7 +603,7 @@ mod tests {
             StyledToken::new("two".to_string(), Style::Plain),
             StyledToken::new("\t".to_string(), Style::Plain),
         ];
-        highlight_nonleading_tabs(&mut tokens);
+        errorlight_nonleading_tabs(&mut tokens);
 
         assert_eq!(
             tokens,
@@ -631,10 +638,10 @@ mod tests {
     }
 
     #[test]
-    fn test_add_trailing_whitespace() {
+    fn test_errorlight_trailing_whitespace() {
         // Just a whitespace
         let mut row = [StyledToken::new(" ".to_string(), Style::Plain)];
-        highlight_trailing_whitespace(&mut row);
+        errorlight_trailing_whitespace(&mut row);
         assert_eq!(row, [StyledToken::new(" ".to_string(), Style::Error)]);
 
         // Trailing whitespace
@@ -642,7 +649,7 @@ mod tests {
             StyledToken::new("x".to_string(), Style::Plain),
             StyledToken::new(" ".to_string(), Style::Plain),
         ];
-        highlight_trailing_whitespace(&mut row);
+        errorlight_trailing_whitespace(&mut row);
         assert_eq!(
             row,
             [
@@ -656,7 +663,7 @@ mod tests {
             StyledToken::new(" ".to_string(), Style::Plain),
             StyledToken::new("x".to_string(), Style::Plain),
         ];
-        highlight_trailing_whitespace(&mut row);
+        errorlight_trailing_whitespace(&mut row);
         assert_eq!(
             row,
             [
@@ -685,7 +692,7 @@ mod tests {
             StyledToken::new("x".to_string(), Style::Plain),
             StyledToken::new("\t".to_string(), Style::Plain),
         ];
-        highlight_nonleading_tabs(&mut row);
+        errorlight_nonleading_tabs(&mut row);
         assert_eq!(
             row,
             [
@@ -700,7 +707,7 @@ mod tests {
             StyledToken::new("\t".to_string(), Style::Plain),
             StyledToken::new("y".to_string(), Style::Plain),
         ];
-        highlight_nonleading_tabs(&mut row);
+        errorlight_nonleading_tabs(&mut row);
         assert_eq!(
             row,
             [
@@ -715,7 +722,7 @@ mod tests {
             StyledToken::new("\t".to_string(), Style::Plain),
             StyledToken::new("x".to_string(), Style::Plain),
         ];
-        highlight_nonleading_tabs(&mut row);
+        errorlight_nonleading_tabs(&mut row);
         assert_eq!(
             row,
             [
@@ -726,7 +733,7 @@ mod tests {
 
         // Single TAB (don't highlight because it is leading)
         let mut row = [StyledToken::new("\t".to_string(), Style::Plain)];
-        highlight_nonleading_tabs(&mut row);
+        errorlight_nonleading_tabs(&mut row);
         assert_eq!(row, [StyledToken::new("\t".to_string(), Style::Plain),]);
     }
 
